
import os, collections
import features


features.Thumbnail_Init()
_lc_handle = features.LocalConn_New(0)
_lc_out_msg_queue = collections.deque()
_lc_in_msg_queue = collections.deque()

class SeriousProblem(Exception): pass

def Thumbnail(source_file, dest_file, image_size):
    rc = features.Thumbnail_Convert(os.path.abspath(source_file),
        os.path.abspath(dest_file), image_size, 24)

    if ( rc == features.THUMBNAIL_SUCCESS ):
        return True

    err_text = 'Thumbnail error: "%s"' % features.Thumbnail_Get_Error(rc)

    if ( rc == features.THUMBNAIL_ERROR_EXTRACT_FAILED ):
        # No thumbnail function for this file type.
        return False

    elif ( rc in (features.THUMBNAIL_ERROR_DIRECTORY, 
                    features.THUMBNAIL_ERROR_FILE_NAME,
                    features.THUMBNAIL_ERROR_SAVE_FAILED) ):
        # File/directory not found
        raise IOError(err_text)

    else:
        # Another problem
        raise SeriousProblem(err_text)

def Send_Message(msg_text):
    if ( len(msg_text) >= features.MAX_SIZE ):
        raise SeriousProblem("Message being sent is too long.")

    _lc_out_msg_queue.append(msg_text)

def Receive_Message():
    if ( len(_lc_in_msg_queue) != 0 ):
        return _lc_in_msg_queue.popleft()
    
    return None

def LocalConn_Poll():

    def Send():
        if ( len(_lc_out_msg_queue) != 0 ):
            msg_text = _lc_out_msg_queue.popleft()

            rc = features.LocalConn_Send(_lc_handle, msg_text)
            if ( rc == 0 ):
                # Sending failed, message still waiting to go out.
                # Put it back in the queue.
                _lc_out_msg_queue.appendleft(msg_text)

    # Send messages (if any)
    Send()

    # Poll
    rc = features.LocalConn_Poll(_lc_handle)
    while ( not ( rc in (
                features.LOCALCONN_ERROR_TEST_ERROR,
                features.LOCALCONN_NOTHING) )):

        if ( rc == features.LOCALCONN_MESSAGE ):
            # Text message received
            _lc_in_msg_queue.append(features.LocalConn_Receive(_lc_handle))
        
        elif ( rc == features.LOCALCONN_SENT_OK ):
            # Can send another mesage now (if there is one)
            Send()

        else:
            # An actual error
            raise SeriousProblem('LocalConnection error: "%s"' %
                    features.LocalConn_Get_Error(rc))
            
        rc = features.LocalConn_Poll(_lc_handle)


